﻿using StockSharp.Algo;
using StockSharp.Algo.Candles;
using StockSharp.Algo.Testing;
using StockSharp.BusinessEntities;
using StockSharp.Messages;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EmulationConnector
{
    class Program
    {

        private static List<Trade> _historyTicks;

        static void Main(string[] args)
        {
            _historyTicks = new List<Trade>();

            var _security = new Security()
            {
                Id = "RIM7@FORTS",
                Board = ExchangeBoard.Forts
            };


            var trade = new Trade
            {
                Id = 1,
                Time = DateTimeOffset.Now.AddMinutes(-5),
                Price = 20,
                Volume = 1,
                OrderDirection = Sides.Buy,
                Security = _security
            };

            _historyTicks.Add(trade);

            var trade1 = new Trade
            {
                Id = 2,
                Time = DateTimeOffset.Now.AddMinutes(-4),
                Price = 21,
                Volume = 1,
                OrderDirection = Sides.Buy,
                Security = _security
            };

            _historyTicks.Add(trade1);

            var trade2 = new Trade
            {
                Id = 3,
                Time = DateTimeOffset.Now.AddMinutes(-3),
                Price = 19,
                Volume = 1,
                OrderDirection = Sides.Buy,
                Security = _security
            };

            _historyTicks.Add(trade2);

            var d1 = _historyTicks.First().Time;
            var d2 = _historyTicks.Last().Time;

            var _historyConnector = new HistoryEmulationConnector(new[] { _security }, new[] { new Portfolio() {Name = "Test" } })
            {
                EmulationAdapter =
                {
                    Emulator = { Settings = { MatchOnTouch = false, } }
                },

                UseExternalCandleSource = false,
                CreateDepthFromOrdersLog = false,
                CreateTradesFromOrdersLog = false,
            };

            _historyConnector.HistoryMessageAdapter.StartDate = d1;
            _historyConnector.HistoryMessageAdapter.StopDate = d2;

            //Func<DateTimeOffset, IEnumerable<Message>> selectionData = d =>

            //_historyTicks.Where(ht => ((ht.ServerTime.Date == d.Date) && (ht.ServerTime.TimeOfDay == d.TimeOfDay)));

            

            var candleManager = new CandleManager(_historyConnector);

            var _candleTimeFrame = TimeSpan.FromMinutes(5);

            var _connectorCandleSеries = new CandleSeries(typeof(TimeFrameCandle), _security, _candleTimeFrame);

            candleManager.Processing += (series, candle) => 
            {
                //TODO
            };


            _historyConnector.NewSecurities += securities =>
            {
                foreach(var security in securities)
                {
                    if (security.Id == _security.Id)
                    {
                        _historyConnector.RegisterHistorySource(_security, MarketDataTypes.Trades, typeof(Trade), GetTrades);
                        _historyConnector.RegisterSecurity(security);
                        _historyConnector.RegisterTrades(security);
                        candleManager.Start(_connectorCandleSеries);
                        _historyConnector.Start();
                    }
                }
            };

            _historyConnector.NewTrades += trades =>
            {
                //TODO
            };

            _historyConnector.Connect();

            Console.Read();

        }

        public static IEnumerable<Message> GetTrades(DateTimeOffset d)
        {
            var trades = _historyTicks.Where(ht => ((ht.Time.Date == d.Date) &&
            (ht.Time.TimeOfDay == d.TimeOfDay))).Select(t => t.ToMessage());
            return trades;
        }


    }
}


